/// Use this class as an annotation for describing how specific fields should
/// be treated by the copyWith code generator
class CopyWithKey {
  /// If true synthetic field will not be taken into account when cloning.
  /// (Synthetic fields are those represented by set and get definitions)
  ///
  /// Default: false
  final bool? ignoreSynthetic;

  /// If true the field will not be passed in constructor, otherwise will be set
  /// directly.
  /// Default: false
  final bool? avoidConstructor;

  /// If true all nullable fields would get the value from copy without any null
  /// validation.
  ///
  /// e.g:
  /// When using allowNullable = false
  /// fieldX = newFieldX != null ? newFieldX : fieldX
  ///
  /// When using allowNullable = true
  /// fieldX = newFieldX
  ///
  /// default: false
  final bool? allowNullable;

  /// Indicates that an empty List, Set, Map or Iterable to be considered
  /// as a null value when deciding copyWith from newData or keep currentData.
  /// Default: true
  final bool? considerEmptyCollectionAsNull;

  /// If true the field will not be taken into account when copyWith
  /// Default: false
  final bool ignore;

  /// If true the field will not be taken into account when copyWith
  /// Default: false
  final bool? includeIfPrivate;

  /// The value to use if the currentData or newData is `null`.
  final Object? defaultValue;

  /// A [Function] to use when copyWith the annotated field.
  ///
  /// Must be a top-level or static [Function] with two positional parameters
  /// compatible with the field being copyWith that returns the same type as the copyWith field.
  /// e.g: Type? function({Type? currentData, Type newData})
  final Function? copyWithFunction;

  const CopyWithKey({
    this.ignoreSynthetic,
    this.allowNullable,
    this.considerEmptyCollectionAsNull,
    bool? ignore,
    this.includeIfPrivate,
    this.avoidConstructor,
    this.defaultValue,
    this.copyWithFunction,
  }) : ignore = ignore ?? false;
}
